package at.bestsolution.rsd.rt.poll;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Consumer;
import java.util.function.Function;

import at.bestsolution.rsd.rt.Subscription;

class FastPollingRegistry implements PollingRegistry {
	private final ScheduledExecutorService executorService;
	private static long POLLING_TIMEOUT;
	
	static {
		try {
			POLLING_TIMEOUT = Long.valueOf(System.getProperty("rsd.fastpolling.frequency", "1000"));	
		} catch( Throwable t ) {
			POLLING_TIMEOUT = 1000;
		}
	}
	
	private final long delay;
	
	public FastPollingRegistry() {
		this.delay = POLLING_TIMEOUT;
		this.executorService = Executors.newScheduledThreadPool(4, code -> {
			Thread t = new Thread(code);
			t.setDaemon(true);
			return t;
		});
	}
	
	
	
	public FastPollingRegistry(long frequence) {
		this.delay = frequence;
		this.executorService = Executors.newScheduledThreadPool(4, code -> {
			Thread t = new Thread(code);
			t.setDaemon(true);
			return t;
		});
	}

	@Override
	public <T> Subscription register(Function<String, PollResult<T>> remoteCall, Consumer<T> consumer) {
		AtomicReference<String> stateMarker = new AtomicReference<>();
		ScheduledFuture<?> future = executorService.scheduleWithFixedDelay( () -> {
			PollResult<T> result = remoteCall.apply(stateMarker.get());
			stateMarker.set(result.stateMarker());
			result.result().forEach(consumer::accept);
		}, 0, delay, TimeUnit.MILLISECONDS);
		return () -> {
			future.cancel(false);
		};
	}

	@Override
	public Subscription registerVoid(Function<String, PollResult<Void>> remoteCall, Runnable consumer) {
		AtomicReference<String> stateMarker = new AtomicReference<>();
		ScheduledFuture<?> future = executorService.scheduleWithFixedDelay( () -> {
			PollResult<Void> result = remoteCall.apply(stateMarker.get());
			stateMarker.set(result.stateMarker());
			consumer.run();
		}, 0, delay, TimeUnit.MILLISECONDS);
		return () -> {
			future.cancel(false);
		};
	}

}
